<?php

namespace Schaffen\Gef\Http\Controllers;

use App\Http\Controllers\Controller;
use App\Http\Utils\Response;
use Illuminate\Http\Request;
use Prettus\Service\Contracts\ServiceInterface;
use Prettus\Service\Exceptions\ServiceException;
use Schaffen\Gef\Http\Requests\CategoriaGastoCreateRequest;
use Schaffen\Gef\Http\Requests\CategoriaGastoUpdateRequest;
use Schaffen\Gef\Models\CategoriaGasto;
use Schaffen\Gef\Repositories\CategoriaGastoRepository;
use Schaffen\Gef\Services\CategoriaGastoService;

/**
 * Class CategoriaGastosController.
 *
 * @package namespace Schaffen\Gef\Http\Controllers;
 */
class CategoriaGastosController extends Controller
{
   /**
    * @var CategoriaGastoRepository
    */
   protected $repository;

   /**
    * @var CategoriaGastoService
    */
   protected $service;

   /**
    * CategoriaGastosController constructor.
    *
    * @param CategoriaGastoRepository $repository
    * @param CategoriaGastoService $service
    */
   public function __construct(CategoriaGastoRepository $repository, CategoriaGastoService $service)
   {
      $this->repository = $repository;
      $this->service = $service;
   }

   /**
    * Display a listing of the resource.
    *
    * @return \Illuminate\Http\Response
    */
   public function index(Request $request)
   {
      $this->repository->pushCriteria(new \Schaffen\Gef\Criterias\RequestPrefixCriteria($request, CategoriaGasto::$prefix));
      $categoria_gasto = $this->repository->all();

      $ret['cols'] = array();
      array_push($ret['cols'], ['field' => "codigo", 'title' => "Código", 'dataType' => 'id', 'filter' => ['codigo' => "text"], 'sortable' => "codigo", 'show' => true]);
      array_push($ret['cols'], ['field' => "tipo", 'title' => "Tipo", 'dataType' => 'text', 'filter' => ['tipo' => "text"], 'sortable' => "tipo", 'show' => true]);
      array_push($ret['cols'], ['field' => "descricao", 'title' => "Descricao", 'dataType' => 'text', 'filter' => ['descricao' => "text"], 'sortable' => "descricao", 'show' => true]);
      array_push($ret['cols'], ['field' => "action", 'title' => "Opções", 'dataType' => 'command', 'show' => true]);
      $ret['rows'] = $categoria_gasto['data'];

      return Response::response(true, '',$ret, []);

   }

   /**
    * Store a newly created resource in storage.
    *
    * @param  CategoriaGastoCreateRequest $request
    *
    * @return \Illuminate\Http\Response
    *
    * @throws \Prettus\Service\Exceptions\ServiceException
    */
   public function store(CategoriaGastoCreateRequest $request)
   {
      $response = $this->service->updateOrCreate($request->all());
      return Response::response($response['status'], $response['message'], $response['data'], $response['errors'], $response['http_code']);
   }

   /**
    * Display the specified resource.
    *
    * @param  int $id
    *
    * @return \Illuminate\Http\Response
    */
   public function show($id)
   {
      $this->repository->setPresenter("Schaffen\\Gef\\Presenters\\CategoriaGastoShowPresenter");
      $categoria_gasto = $this->repository->find($id);


      return Response::response(true, null, $categoria_gasto, null);
   }

   /**
    * Update the specified resource in storage.
    *
    * @param  CategoriaGastoUpdateRequest $request
    * @param  string $id
    *
    * @return Response
    *
    * @throws \Prettus\Service\Exceptions\ServiceException
    */
   public function update(CategoriaGastoUpdateRequest $request, $id)
   {
      $response = $this->service->updateOrCreate($request->all(), $id);
      return Response::response($response['status'], $response['message'], $response['data'], $response['errors'], $response['http_code']);
   }


   /**
    * Remove the specified resource from storage.
    *
    * @param  int $id
    *
    * @return \Illuminate\Http\Response
    */
   public function destroy($id)
   {
      $deleted = $this->repository->delete($id);

      if (request()->wantsJson()) {

         return response()->json([
            'message' => 'CategoriaGasto deleted.',
            'deleted' => $deleted,
         ]);
      }

      return redirect()->back()->with('message', 'CategoriaGasto deleted.');
   }

   public function categoriaTipo()
   {

      $tipo = CategoriaGasto::$tipos;
      return Response::response(true,null, $tipo,null);
   }
}
