<?php

namespace Schaffen\Gef\Services;

use App\Http\Utils\SCHArray;
use Carbon\Carbon;
use Prettus\Validator\Exceptions\ValidatorException;
use Schaffen\Gef\Criterias\AgendamentosDentroPeriodoCriteria;
use Schaffen\Gef\Models\AgendamentoVeiculoUso;
use Schaffen\Gef\Repositories\AgendamentoVeiculoUsoRepository;
use Schaffen\Gef\Validators\AgendamentoVeiculoUsoValidator;

class AgendamentoVeiculoUsoService
{

   private $repository;
   private $validator;

   public function __construct(AgendamentoVeiculoUsoRepository $repository, AgendamentoVeiculoUsoValidator $validator)
   {
      $this->repository = $repository;
      $this->validator = $validator;
   }

   public function updateOrCreate($data, $codigo = null)
   {
      try {

         $validator_rules = is_numeric($codigo) ? AgendamentoVeiculoUsoValidator::RULE_UPDATE : AgendamentoVeiculoUsoValidator::RULE_CREATE;
         $this->validator->with($data)->passesOrFail($validator_rules);
         $this->repository->setPresenter("Schaffen\\Gef\\Presenters\\AgendamentoVeiculoUsoPresenter");

         $data_disponibilidade_inicial_carbon = Carbon::createFromFormat('d/m/Y',$data['data_disponibilidade_inicial']);
         $data_disponibilidade_final_carbon = Carbon::createFromFormat('d/m/Y',$data['data_disponibilidade_final']);

         $attrib = ['agu_codigo' => $codigo];
         if (!is_numeric($codigo)) {
            $agendamento_periodo = Collect($this->repository->getByCriteria(new AgendamentosDentroPeriodoCriteria($data['motorista'],
               $data['veiculo'],
               $data_disponibilidade_inicial_carbon,
               $data_disponibilidade_final_carbon))['data'])->first();
            if (isset($agendamento_periodo)) {
               throw new \Exception("Já exite um agendamento com esses dados");
            }

         }

         $data['fk_motorista_pes_codigo'] = $data['motorista'];
         $data['fk_vei_codigo'] = $data['veiculo'];
         $data = SCHArray::addPrefix(AgendamentoVeiculoUso::$prefix, $data);

         $agendamento_uso = $this->repository->updateOrCreate($attrib, $data);

         $response = [
            'status' => true,
            'errors' => '',
            'message' => 'Agendamento de uso do veículo gravado.',
            'data' => $agendamento_uso['data'],
            'http_code' => '200'
         ];


         return $response;

      } catch (ValidatorException $e) {
         return $response = [
            'status' => false,
            'errors' => $e->getMessageBag(),
            'message' => 'Não foi possível gravar o Agendamento de uso do veículo',
            'data' => null,
            'http_code' => '400'
         ];
      } catch (\Exception $e) {
         return $response = [
            'status' => false,
            'errors' => $e->getMessage(),
            'message' => 'Não foi possível gravar o Agendamento de uso do veículo',
            'data' => null,
            'http_code' => '400'
         ];
      }
   }
}
